@@ -2,6 +2,8 @@ require 'spreadsheet/encodings'
 require 'spreadsheet/font'
 require 'spreadsheet/formula'
 require 'spreadsheet/link'
+require 'spreadsheet/note'
+require 'spreadsheet/noteObject'
 require 'spreadsheet/excel/error'
 require 'spreadsheet/excel/internals'
 require 'spreadsheet/excel/sst_entry'
@@ -113,6 +115,17 @@ class Reader
     end
   end
   def postread_worksheet worksheet
+     #We now have a lot of Note and NoteObjects, but they're not linked
+     #So link the noteObject(text) to the note (with author, position)
+     #TODO
+     @noteList.each do |i|
+        matching_obj = @noteObjList.select {|j| j.objID == i.objID}
+        if matching_obj.length > 1
+           puts "ERROR - more than one matching object ID!"
+        end
+        i.text = matching_obj.first.text
+        worksheet.add_note i.row, i.col, i.text
+     end
   end
   ##
   # The entry-point for reading Excel-documents. Reads the Biff-Version and
@@ -178,7 +191,7 @@ class Reader
     #      6  var.  Sheet name: BIFF5/BIFF7: Byte string,
     #                           8-bit string length (➜ 3.3)
     #                           BIFF8: Unicode string, 8-bit string length (➜ 3.4)
-    offset, _, _ = work.unpack("VC2")
+    offset, visibility, _ = work.unpack("VC2")
     name = client read_string(work[6..-1]), @workbook.encoding
     if @boundsheets
       @boundsheets[0] += 1
@@ -190,7 +203,8 @@ class Reader
     @workbook.add_worksheet Worksheet.new(:name     => name,
                                           :ole      => @book,
                                           :offset   => offset,
-                                          :reader   => self)
+                                          :reader   => self,
+                                          :visibility => WORKSHEET_VISIBILITIES[visibility])
   end
   def read_codepage work, pos, len
     codepage, _ = work.unpack 'v'
@@ -829,6 +843,8 @@ class Reader
   def read_worksheet worksheet, offset
     @pos = offset
     @detected_rows = {}
+    @noteObjList = []
+    @noteList = []
     previous = nil
     while tuple = get_next_chunk
       pos, op, len, work = tuple
@@ -866,6 +882,42 @@ class Reader
         read_merged_cells worksheet, work, pos, len
       when :protect, :password
         read_sheet_protection worksheet, op, work
+      when :note # a note references an :obj
+        read_note worksheet, work, pos, len
+      when :obj # it contains the author in the NTS structure
+        _ft, _cb, _ot, _objID = work.unpack('v4')
+        if _ot == 0x19
+          #puts "\nDEBUG: found Note Obj record"
+          @noteObject         = NoteObject.new
+          @noteObject.objID   = _objID
+        end
+        #p work
+      when :drawing # this can be followed by txo in case of a note
+        if previous == :obj
+          #puts "\nDEBUG: found MsDrawing record"
+          #p work
+        end
+      when :txo # this contains the length of the note text
+        if previous == :drawing
+          #puts "\nDEBUG: found TxO record"
+          #p work
+        end
+      when :continue # this contains the actual note text
+        if previous == :txo && @noteObject
+          #puts "\nDEBUG: found Continue record"
+          continueFmt = work.unpack('C')
+          if (continueFmt.first == 0)
+             #puts "Picking compressed charset"
+             #Skip to offset due to 'v5C' used above
+             _text = work.unpack('@1C*')
+             @noteObject.text = _text.pack('C*')
+          elsif (continueFmt.first == 1)
+             #puts "Picking uncompressed charset"
+             _text = work.unpack('@1S*')
+             @noteObject.text = _text.pack('U*')
+          end
+          @noteObjList << @noteObject
+        end
       when :pagesetup
         read_pagesetup(worksheet, work, pos, len)
       when :leftmargin
@@ -881,7 +933,8 @@ class Reader
           set_missing_row_address worksheet, work, pos, len
         end
       end
-      previous = op
+      previous = op 
+      #previous = op unless op == :continue
     end
   end
 
@@ -1086,6 +1139,31 @@ class Reader
     fmt.pattern_bg_color = COLOR_CODES[(xf_pattern & 0x3f80) >> 7] || :pattern_bg
     @workbook.add_format fmt
   end
+  def read_note worksheet, work, pos, len
+    #puts "\nDEBUG: found a note record in read_worksheet\n"
+    row, col, _, _objID, _objAuthLen, _objAuthLenFmt = work.unpack('v5C')
+    if (_objAuthLen > 0)
+       if (_objAuthLenFmt == 0)
+          #puts "Picking compressed charset"
+          #Skip to offset due to 'v5C' used above
+          _objAuth = work.unpack('@11C' + (_objAuthLen-1).to_s + 'C')
+       elsif (_objAuthLenFmt == 1)
+          #puts "Picking uncompressed charset"
+          _objAuth = work.unpack('@11S' + (_objAuthLen-1).to_s + 'S')
+       end
+       _objAuth = _objAuth.pack('C*')
+    else
+       _objAuth = ""
+    end
+    @note = Note.new
+    @note.length = len
+    @note.row    = row
+    @note.col    = col
+    @note.author = _objAuth
+    @note.objID  = _objID
+    #Pop it on the list to be sorted in postread_worksheet
+    @noteList << @note
+  end
   def read_sheet_protection worksheet, op, data
     case op
     when :protect
